@@ -24,10 +24,10 @@
  */
 
 #include <unistd.h>
+#include <string.h>
 #include <libbonobo.h>
 #include <glib/gmain.h>
 #include <gnome-speech/gnome-speech.h>
-#include <eci.h>
 #include "viavoicesynthesisdriver.h"
 #include "viavoicespeaker.h"
 
@@ -46,7 +46,7 @@ viavoice_speaker_from_servant (PortableServer_Servant *servant)
 static void
 viavoice_add_parameter (ECIHand handle,
 			ViavoiceSpeaker *viavoice_speaker,
-			ECIVoiceParam param,
+			enum ECIVoiceParam param,
 			const gchar *parameter_name,
 			gdouble min,
 			gdouble max,
@@ -56,6 +56,16 @@ viavoice_add_parameter (ECIHand handle,
 	gdouble current;
 	current = (gdouble)
 		eciGetVoiceParam (handle, 0, param);
+
+	/* normalize for real world mode */
+	if (eciSpeed == param) {
+		current = current * 5 + 70;
+		current = (current > max ? max : current);
+	} else if (eciPitchBaseline == param) {
+		current = current * 4 + 40;
+		current = (current > max ? max : current);
+	}
+
 	speaker_add_parameter (speaker,
 			       parameter_name,
 			       min,
@@ -66,6 +76,23 @@ viavoice_add_parameter (ECIHand handle,
 
 
 
+static void
+viavoice_add_insertindex (ECIHand handle,
+			ViavoiceSpeaker *viavoice_speaker,
+			const gchar *parameter_name,
+			gdouble min,
+			gdouble max,
+			parameter_set_func func)
+{
+	Speaker *speaker = SPEAKER(viavoice_speaker);
+	speaker_add_parameter (speaker,
+			       parameter_name,
+			       min,
+			       -1, /* prevent reentrance during add */
+			       max,
+			       func);
+}
+
 static CORBA_boolean
 viavoice_registerSpeechCallback (PortableServer_Servant servant,
 				 const GNOME_Speech_SpeechCallback callback,
@@ -96,6 +123,27 @@ viavoice_say (PortableServer_Servant servant,
 }
 
 
+static void
+viavoice_add_device_parameter (ECIHand handle,
+			ViavoiceSpeaker *viavoice_speaker,
+			enum ECIParam param,
+			const gchar *parameter_name,
+			gdouble min,
+			gdouble max,
+			parameter_set_func func)
+{
+	Speaker *speaker = SPEAKER(viavoice_speaker);
+	gdouble current;
+	current = (gdouble)
+		eciGetParam (handle, param);
+	speaker_add_parameter (speaker,
+			       parameter_name,
+			       min,
+			       current,
+			       max,
+			       func);
+}
+
 
 static CORBA_boolean
 viavoice_stop (PortableServer_Servant servant,
@@ -209,8 +257,10 @@ viavoice_set_pitch (Speaker *speaker,
 		    gdouble new_pitch)
 {
 	ViavoiceSynthesisDriver *d = VIAVOICE_SYNTHESIS_DRIVER (speaker->driver);
+	int normalize = 4; /* normalize eci value to real world */
 
-	viavoice_synthesis_driver_set_voice_param (d, eciPitchBaseline, (int) new_pitch);
+	viavoice_synthesis_driver_set_voice_param (d, eciPitchBaseline, 
+		(int) new_pitch / normalize);
 	return TRUE;
 }
 
@@ -257,8 +307,10 @@ viavoice_set_rate (Speaker *speaker,
 		   gdouble new_rate)
 {
 	ViavoiceSynthesisDriver *d = VIAVOICE_SYNTHESIS_DRIVER (speaker->driver);
+	int normalize = 5; /* normalize eci value to real world */
 
-	viavoice_synthesis_driver_set_voice_param (d, eciSpeed, (int) new_rate);
+	viavoice_synthesis_driver_set_voice_param (d, eciSpeed, 
+		(int) new_rate / normalize);
 	return TRUE;
 }
 
@@ -275,6 +327,100 @@ viavoice_set_volume (Speaker *speaker,
 }
 
 
+static gboolean
+viavoice_set_dictionary (Speaker *speaker,
+		     gdouble new_dictionary)
+{
+	ViavoiceSynthesisDriver *d = VIAVOICE_SYNTHESIS_DRIVER (speaker->driver);
+
+	viavoice_synthesis_driver_set_device_param (d, eciDictionary, (int) new_dictionary);
+	return TRUE;
+}
+
+
+static gboolean
+viavoice_set_numbermode (Speaker *speaker,
+		     gdouble new_numbermode)
+{
+	ViavoiceSynthesisDriver *d = VIAVOICE_SYNTHESIS_DRIVER (speaker->driver);
+
+	viavoice_synthesis_driver_set_device_param (d, eciNumberMode, (int) new_numbermode);
+	return TRUE;
+}
+
+
+static gboolean
+viavoice_set_samplerate (Speaker *speaker,
+		     gdouble new_samplerate)
+{
+	ViavoiceSynthesisDriver *d = VIAVOICE_SYNTHESIS_DRIVER (speaker->driver);
+
+	viavoice_synthesis_driver_set_device_param (d, eciSampleRate, (int) new_samplerate);
+	return TRUE;
+}
+
+static gboolean
+viavoice_set_wanttextmode (Speaker *speaker,
+		     gdouble new_wanttextmode)
+{
+	ViavoiceSynthesisDriver *d = VIAVOICE_SYNTHESIS_DRIVER (speaker->driver);
+
+	viavoice_synthesis_driver_set_device_param (d, eciTextMode, (int) new_wanttextmode);
+	return TRUE;
+}
+
+static gboolean
+viavoice_set_wantwordindex (Speaker *speaker,
+		     gdouble new_wantwordindex)
+{
+	ViavoiceSynthesisDriver *d = VIAVOICE_SYNTHESIS_DRIVER (speaker->driver);
+
+	viavoice_synthesis_driver_set_device_param (d, eciWantWordIndex, (int) new_wantwordindex);
+	return TRUE;
+}
+
+static gboolean
+viavoice_set_wantphonemeindices (Speaker *speaker,
+		     gdouble new_wantphonemeindices)
+{
+	ViavoiceSynthesisDriver *d = VIAVOICE_SYNTHESIS_DRIVER (speaker->driver);
+
+	viavoice_synthesis_driver_set_device_param (d, eciWantPhonemeIndices, (int) new_wantphonemeindices);
+	return TRUE;
+}
+
+static gboolean
+viavoice_set_synthmode (Speaker *speaker,
+		     gdouble new_synthmode)
+{
+	ViavoiceSynthesisDriver *d = VIAVOICE_SYNTHESIS_DRIVER (speaker->driver);
+
+	viavoice_synthesis_driver_set_device_param (d, eciSynthMode, (int) new_synthmode);
+	return TRUE;
+}
+
+static gboolean
+viavoice_set_insertindex (Speaker *speaker,
+		     gdouble new_index)
+{
+	static gboolean rc;
+	char buf[BUFSIZ];
+
+	ViavoiceSynthesisDriver *d = VIAVOICE_SYNTHESIS_DRIVER (speaker->driver);
+
+	if (new_index < 0) {
+		return TRUE;	/* prevent reentrance during add */
+	}
+
+	rc = eciInsertIndex (d, (int) new_index);
+
+	if (!rc) {
+		eciErrorMessage(d, buf);
+		fprintf(stderr, "viavoice_set_insertindex(): %s\n", buf);
+	}
+
+	return rc;
+}
 
 ViavoiceSpeaker *
 viavoice_speaker_new (GObject *d,
@@ -282,25 +428,26 @@ viavoice_speaker_new (GObject *d,
 {
 	ViavoiceSpeaker *speaker;
 	Speaker *s;
-	ECIHand handle;
+	ViavoiceSynthesisDriver *a_driver=(ViavoiceSynthesisDriver*)d;
+	ECIHand handle = a_driver->handle;
 	gint i;
 	gint voice_num = 1;
 	speaker = g_object_new (VIAVOICE_SPEAKER_TYPE, NULL);
 	s = SPEAKER(speaker);
-
-	/* Create a handle and initialize it */
-
-	handle = eciNew ();
+	enum ECILanguageDialect dialect;
 
 	s->driver = g_object_ref (d);
 
         /* Find the specified voice */
 
+	dialect = viavoice_synthesis_driver_language_to_dialect(voice_spec->language);
+	eciSetParam(handle, eciLanguageDialect, dialect);
+
 	for (i = 1; i <= ECI_PRESET_VOICES; i++) {
 		char name[30];
 
 		eciGetVoiceName (handle, i, name);
-		if (!g_strcasecmp (voice_spec->name, name))
+		if (0 == g_ascii_strcasecmp (voice_spec->name, name))
 			voice_num = i;
 	}
 
@@ -310,6 +457,54 @@ viavoice_speaker_new (GObject *d,
 
         /* Add parameters */
 
+	viavoice_add_insertindex (handle, speaker,
+				"insert index",
+				0,
+				INT_MAX,
+				viavoice_set_insertindex);
+
+	viavoice_add_device_parameter (handle, speaker,
+				eciTextMode,
+				"text mode",
+				0,
+				3,
+				viavoice_set_wanttextmode);  
+	viavoice_add_device_parameter (handle, speaker,
+				eciWantWordIndex,
+				"want word index",
+				0,
+				1,
+				viavoice_set_wantwordindex);  
+	viavoice_add_device_parameter (handle, speaker,
+				eciWantPhonemeIndices,
+				"want phoneme indices",
+				0,
+				1,
+				viavoice_set_wantphonemeindices);  
+	viavoice_add_device_parameter (handle, speaker,
+				eciSynthMode,
+				"synthesis mode",
+				0,
+				1,
+				viavoice_set_synthmode);  
+	viavoice_add_device_parameter (handle, speaker,
+				eciSampleRate,
+				"sample rate",
+				0,
+				2,
+				viavoice_set_samplerate);  
+	viavoice_add_device_parameter (handle, speaker,
+				eciNumberMode,
+				"number mode",
+				0,
+				1,
+				viavoice_set_numbermode);  
+	viavoice_add_device_parameter (handle, speaker,
+				eciDictionary,
+				"dictionary",
+				0,
+				1,
+				viavoice_set_dictionary);  
 	viavoice_add_parameter (handle, speaker,
 				eciGender,
 				"gender",
@@ -325,8 +520,8 @@ viavoice_speaker_new (GObject *d,
 	viavoice_add_parameter (handle, speaker,
 				eciPitchBaseline,
 				"pitch",
-				0,
-				100,
+				40,	/* real world */
+				422,	/* real world */
 				viavoice_set_pitch);  
 	viavoice_add_parameter (handle, speaker,
 				eciPitchFluctuation,
@@ -349,8 +544,8 @@ viavoice_speaker_new (GObject *d,
 	viavoice_add_parameter (handle, speaker,
 				eciSpeed,
 				"rate",
-				0,
-				100,
+				70,	/* real world */
+				1297,	/* real world */
 				viavoice_set_rate);  
 	viavoice_add_parameter (handle, speaker,
 				eciVolume,
@@ -359,10 +554,6 @@ viavoice_speaker_new (GObject *d,
 				100,
 				viavoice_set_volume);  
 
-	eciDelete (handle);
 	return speaker;
 }
 
-
-
-
